@@ -3,6 +3,8 @@
 # Author: modelf
 ###############################################################################
 
+rv <- R.Version()
+if(as.numeric(rv$major)>3 | (as.numeric(rv$major)==3 & as.numeric(rv$minor)>=6.0)) RNGkind(sample.kind = "Rounding")
 library("RUnit")
 library(mcr)
 
